@@ -50,24 +50,54 @@ importance.ranger <- function(x, ...) {
   return(x$variable.importance)
 }
 
-##' Compute variable importance with p-values.
-##'
+##' Compute variable importance with p-values. 
+##' For high dimensional data, the fast method of Janitza et al. (2016) can be used. 
+##' The permutation approach of Altmann et al. (2010) is computationally intensive but can be used with all kinds of data. 
+##' See below for details. 
+##' 
+##' The method of Janitza et al. (2016) uses a clever trick:
+##' With an unbiased variable importance measure, the importance values of non-associated variables vary randomly around zero. 
+##' Thus, all non-positive importance values are assumed to correspond to these non-associated variables and they are used to construct a distribution of the importance under the null hypothesis of no association to the response.
+##' Since only the non-positive values of this distribution can be observed, the positive values are created by mirroring the negative distribution. 
+##' See Janitza et al. (2016) for details.
+##' 
+##' The method of Altmann et al. (2010) uses a simple permutation test: 
+##' The distribution of the importance under the null hypothesis of no association to the response is created by several replications of permuting the response, growing an RF and computing the variable importance.
+##' The authors recommend 50-100 permutations. 
+##' However, much larger numbers have to be used to estimate more precise p-values.
+##' We add 1 to the numerator and denominator to avoid zero p-values.
 ##'
 ##' @title ranger variable importance p-values
-##' @param x ranger or holdoutRF object.
-##' @param method Method to compute p-values. Use "janitza" for the method by Janitza et al. (2015) or "altmann" for the non-parametric method by Altmann et al. (2010).
+##' @param x \code{ranger} or \code{holdoutRF} object.
+##' @param method Method to compute p-values. Use "janitza" for the method by Janitza et al. (2016) or "altmann" for the non-parametric method by Altmann et al. (2010).
 ##' @param num.permutations Number of permutations. Used in the "altmann" method only.
 ##' @param formula Object of class formula or character describing the model to fit. Used in the "altmann" method only.
 ##' @param data Training data of class data.frame or matrix. Used in the "altmann" method only.
-##' @param ... Further arguments passed to ranger(). Used in the "altmann" method only.
-##' @return Variable importance and p-values.
+##' @param ... Further arguments passed to \code{ranger()}. Used in the "altmann" method only.
+##' @return Variable importance and p-value for each variable.
+##' @examples
+##' require(ranger)
+##' 
+##' ## Janitza's p-values with corrected Gini importance
+##' n <- 50
+##' p <- 400
+##' dat <- data.frame(y = factor(rbinom(n, 1, .5)), replicate(p, runif(n)))
+##' rf.sim <- ranger(y ~ ., dat, importance = "impurity_corrected")
+##' importance_pvalues(rf.sim, method = "janitza")
+##' 
+##' ## Permutation p-values 
+##' \dontrun{
+##' rf.iris <- ranger(Species ~ ., data = iris, importance = 'permutation')
+##' importance_pvalues(rf.iris, method = "altmann", formula = Species ~ ., data = iris)
+##' }
 ##' @seealso \code{\link{ranger}}
 ##' @author Marvin N. Wright
 ##' @references
-##'   Janitza, S., Celik, E. & Boulesteix, A.-L., (2015). A computationally fast variable importance test for random forests for high-dimensional data. Adv Data Anal Classif \url{http://dx.doi.org/10.1007/s11634-016-0276-4}. \cr
-##'   Altmann, A., Tolosi, L., Sander, O. & Lengauer, T. (2010). Permutation importance: a corrected feature importance measure, Bioinformatics 26(10):1340-1347.
+##'   Janitza, S., Celik, E. & Boulesteix, A.-L., (2016). A computationally fast variable importance test for random forests for high-dimensional data. Adv Data Anal Classif \url{http://dx.doi.org/10.1007/s11634-016-0276-4}. \cr
+##'   Altmann, A., Tolosi, L., Sander, O. & Lengauer, T. (2010). Permutation importance: a corrected feature importance measure, Bioinformatics 26:1340-1347.
 ##' @export 
 importance_pvalues <- function(x, method = c("janitza", "altmann"), num.permutations = 100, formula = NULL, data = NULL, ...) {
+  method <- match.arg(method)
   if (class(x) != "ranger" & class(x) != "holdoutRF") {
     stop("Object is no ranger or holdoutRF object.")
   }
@@ -77,10 +107,10 @@ importance_pvalues <- function(x, method = c("janitza", "altmann"), num.permutat
 
   if (method == "janitza") {
     if (x$importance.mode == "impurity") {
-      stop("Impurity variable importance found. Please use (hold-out) permutation importance to use this method.")
+      stop("Impurity variable importance found. Please use (hold-out) permutation importance or corrected impurity importance to use this method.")
     }
     if (class(x) != "holdoutRF" && x$importance.mode == "permutation") {
-      warning("Permutation variable importance found, inaccurate p-values. Please use hold-out permutation importance to use this method.")
+      warning("Permutation variable importance found, inaccurate p-values. Please use hold-out permutation importance or corrected impurity importance to use this method.")
     }
     if (x$treetype != "Classification") {
       warning("This method is tested for classification only, use with care.")
@@ -92,7 +122,8 @@ importance_pvalues <- function(x, method = c("janitza", "altmann"), num.permutat
     vimp <- c(m1, -m1, m2)
     
     ## Compute p-value
-    pval <- 1 - ecdf(vimp)(x$variable.importance)
+    ## Note: ecdf is smaller or equal, problems with 0 importance values
+    pval <- 1 - numSmaller(x$variable.importance, vimp) / length(vimp)
     
     ## TODO: 100 ok? increase? 
     if (length(m1) == 0) {
@@ -115,11 +146,11 @@ importance_pvalues <- function(x, method = c("janitza", "altmann"), num.permutat
     } else {
       dependent.variable.name <- all.vars(formula)[1]
     }
-    vimp <- replicate(num.permutations, {
+    vimp <- sapply(1:num.permutations, function(i) {
       dat <- data
       dat[, dependent.variable.name] <- dat[sample(nrow(dat)), dependent.variable.name]
       ranger(formula, dat, num.trees = x$num.trees, mtry = x$mtry, min.node.size = x$min.node.size, 
-             importance = x$importance.mode, ...)$variable.importance
+             importance = x$importance.mode, replace = x$replace, ...)$variable.importance
     })
     
     ## Compute p-value
