@@ -31,6 +31,9 @@
 
 #include <vector>
 #include <iostream>
+#include <numeric>
+#include <random>
+#include <algorithm>
 
 #include "globals.h"
 
@@ -57,6 +60,12 @@ public:
   void getMinMaxValues(double& min, double&max, std::vector<size_t>& sampleIDs, size_t varID);
 
   size_t getIndex(size_t row, size_t col) const {
+    // Use permuted data for corrected impurity importance
+    if (col >= num_cols) {
+      col = getUnpermutedVarID(col);
+      row = getPermutedSampleID(row);
+    }
+
     if (col < num_cols_no_snp) {
       return index_data[col * num_rows + row];
     } else {
@@ -74,6 +83,11 @@ public:
   }
 
   double getUniqueDataValue(size_t varID, size_t index) const {
+    // Use permuted data for corrected impurity importance
+    if (varID >= num_cols) {
+      varID = getUnpermutedVarID(varID);
+    }
+
     if (varID < num_cols_no_snp) {
       return unique_data_values[varID][index];
     } else {
@@ -83,6 +97,11 @@ public:
   }
 
   size_t getNumUniqueDataValues(size_t varID) const {
+    // Use permuted data for corrected impurity importance
+    if (varID >= num_cols) {
+      varID = getUnpermutedVarID(varID);
+    }
+
     if (varID < num_cols_no_snp) {
       return unique_data_values[varID].size();
     } else {
@@ -113,6 +132,62 @@ public:
     }
   }
 
+  std::vector<size_t>& getNoSplitVariables() {
+    return no_split_variables;
+  }
+
+  void addNoSplitVariable(size_t varID) {
+    no_split_variables.push_back(varID);
+    std::sort(no_split_variables.begin(), no_split_variables.end());
+  }
+
+  std::vector<bool>& getIsOrderedVariable() {
+    return is_ordered_variable;
+  }
+
+  void setIsOrderedVariable(std::vector<std::string>& unordered_variable_names) {
+    is_ordered_variable.resize(num_cols, true);
+    for (auto& variable_name : unordered_variable_names) {
+      size_t varID = getVariableID(variable_name);
+      is_ordered_variable[varID] = false;
+    }
+  }
+
+  void setIsOrderedVariable(std::vector<bool>& is_ordered_variable) {
+    this->is_ordered_variable = is_ordered_variable;
+  }
+
+  const bool isOrderedVariable(size_t varID) const {
+    // Use permuted data for corrected impurity importance
+    if (varID >= num_cols) {
+      varID = getUnpermutedVarID(varID);
+    }
+    return is_ordered_variable[varID];
+  }
+
+  void permuteSampleIDs(std::mt19937_64 random_number_generator) {
+    permuted_sampleIDs.resize(num_rows);
+    std::iota(permuted_sampleIDs.begin(), permuted_sampleIDs.end(), 0);
+    std::shuffle(permuted_sampleIDs.begin(), permuted_sampleIDs.end(), random_number_generator);
+  }
+
+  const size_t getPermutedSampleID(size_t sampleID) const {
+    return permuted_sampleIDs[sampleID];
+  }
+
+  const size_t getUnpermutedVarID(size_t varID) const {
+    if (varID >= num_cols) {
+      varID -= num_cols;
+
+      for (auto& skip : no_split_variables) {
+        if (varID >= skip) {
+          ++varID;
+        }
+      }
+    }
+    return varID;
+  }
+
 protected:
   std::vector<std::string> variable_names;
   size_t num_rows;
@@ -128,6 +203,15 @@ protected:
   std::vector<std::vector<double>> unique_data_values;
   size_t max_num_unique_values;
 
+  // Variable to not split at (only dependent_varID for non-survival trees)
+  std::vector<size_t> no_split_variables;
+
+  // For each varID true if ordered
+  std::vector<bool> is_ordered_variable;
+
+  // Permuted samples for corrected impurity importance
+  std::vector<size_t> permuted_sampleIDs;
+
 private:
   DISALLOW_COPY_AND_ASSIGN(Data);
 };
