@@ -42,18 +42,18 @@ public:
 
   // Create from loaded forest
   Tree(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
-      std::vector<double>& split_values, std::vector<bool>* is_ordered_variable);
+      std::vector<double>& split_values);
 
   virtual ~Tree();
 
   void init(Data* data, uint mtry, size_t dependent_varID, size_t num_samples, uint seed,
       std::vector<size_t>* deterministic_varIDs, std::vector<size_t>* split_select_varIDs,
       std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size,
-      std::vector<size_t>* no_split_variables, bool sample_with_replacement, std::vector<bool>* is_unordered,
-      bool memory_saving_splitting, SplitRule splitrule, std::vector<double>* case_weights, bool keep_inbag,
-      double sample_fraction, double alpha, double minprop, bool holdout, uint num_random_splits);
+      bool sample_with_replacement, bool memory_saving_splitting, SplitRule splitrule,
+      std::vector<double>* case_weights, bool keep_inbag, double sample_fraction, double alpha, double minprop,
+      bool holdout, uint num_random_splits);
 
-  virtual void initInternal() = 0;
+  virtual void allocateMemory() = 0;
 
   void grow(std::vector<double>* variable_importance);
 
@@ -116,12 +116,6 @@ protected:
   // Number of OOB samples
   size_t num_samples_oob;
 
-  // For each varID true if unordered
-  std::vector<bool>* is_ordered_variable;
-
-  // Variable to not split at (only dependent_varID for non-survival trees)
-  std::vector<size_t>* no_split_variables;
-
   // Minimum node size to split, like in original RF nodes of smaller size can be produced
   uint min_node_size;
 
