@@ -46,17 +46,16 @@ TreeSurvival::TreeSurvival(std::vector<double>* unique_timepoints, size_t status
 
 TreeSurvival::TreeSurvival(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
     std::vector<double>& split_values, std::vector<std::vector<double>> chf, std::vector<double>* unique_timepoints,
-    std::vector<size_t>* response_timepointIDs, std::vector<bool>* is_ordered_variable) :
-    Tree(child_nodeIDs, split_varIDs, split_values, is_ordered_variable), status_varID(0), unique_timepoints(
-        unique_timepoints), response_timepointIDs(response_timepointIDs), chf(chf), num_deaths(0), num_samples_at_risk(
-        0) {
+    std::vector<size_t>* response_timepointIDs) :
+    Tree(child_nodeIDs, split_varIDs, split_values), status_varID(0), unique_timepoints(unique_timepoints), response_timepointIDs(
+        response_timepointIDs), chf(chf), num_deaths(0), num_samples_at_risk(0) {
   this->num_timepoints = unique_timepoints->size();
 }
 
 TreeSurvival::~TreeSurvival() {
 }
 
-void TreeSurvival::initInternal() {
+void TreeSurvival::allocateMemory() {
   // Number of deaths and samples at risk for each timepoint
   num_deaths = new size_t[num_timepoints];
   num_samples_at_risk = new size_t[num_timepoints];
@@ -100,7 +99,7 @@ double TreeSurvival::computePredictionAccuracyInternal() {
   std::vector<double> sum_chf;
   for (size_t i = 0; i < prediction_terminal_nodeIDs.size(); ++i) {
     size_t terminal_nodeID = prediction_terminal_nodeIDs[i];
-    sum_chf.push_back(std::accumulate(chf[terminal_nodeID].begin(), chf[terminal_nodeID].end(), 0));
+    sum_chf.push_back(std::accumulate(chf[terminal_nodeID].begin(), chf[terminal_nodeID].end(), 0.0));
   }
 
   // Return concordance index
@@ -134,7 +133,7 @@ bool TreeSurvival::findBestSplit(size_t nodeID, std::vector<size_t>& possible_sp
     for (auto& varID : possible_split_varIDs) {
 
       // Find best split value, if ordered consider all values as split values, else all 2-partitions
-      if ((*is_ordered_variable)[varID]) {
+      if (data->isOrderedVariable(varID)) {
         if (splitrule == LOGRANK) {
           findBestSplitValueLogRank(nodeID, varID, best_value, best_varID, best_decrease);
         } else if (splitrule == AUC || splitrule == AUC_IGNORE_TIES) {
@@ -155,6 +154,12 @@ bool TreeSurvival::findBestSplit(size_t nodeID, std::vector<size_t>& possible_sp
     // If not terminal node save best values
     split_varIDs[nodeID] = best_varID;
     split_values[nodeID] = best_value;
+
+    // Compute decrease of impurity for this node and add to variable importance if needed
+    if (importance_mode == IMP_GINI || importance_mode == IMP_GINI_CORRECTED) {
+      addImpurityImportance(nodeID, best_varID, best_decrease);
+    }
+
     return false;
   }
 }
@@ -661,7 +666,7 @@ bool TreeSurvival::findBestSplitExtraTrees(size_t nodeID, std::vector<size_t>& p
     for (auto& varID : possible_split_varIDs) {
 
       // Find best split value, if ordered consider all values as split values, else all 2-partitions
-      if ((*is_ordered_variable)[varID]) {
+      if (data->isOrderedVariable(varID)) {
         findBestSplitValueExtraTrees(nodeID, varID, best_value, best_varID, best_decrease);
       } else {
         findBestSplitValueExtraTreesUnordered(nodeID, varID, best_value, best_varID, best_decrease);
@@ -678,6 +683,12 @@ bool TreeSurvival::findBestSplitExtraTrees(size_t nodeID, std::vector<size_t>& p
     // If not terminal node save best values
     split_varIDs[nodeID] = best_varID;
     split_values[nodeID] = best_value;
+
+    // Compute decrease of impurity for this node and add to variable importance if needed
+    if (importance_mode == IMP_GINI || importance_mode == IMP_GINI_CORRECTED) {
+      addImpurityImportance(nodeID, best_varID, best_decrease);
+    }
+
     return false;
   }
 }
@@ -889,3 +900,20 @@ void TreeSurvival::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t v
   }
 }
 
+void TreeSurvival::addImpurityImportance(size_t nodeID, size_t varID, double decrease) {
+
+  // No variable importance for no split variables
+  size_t tempvarID = data->getUnpermutedVarID(varID);
+  for (auto& skip : data->getNoSplitVariables()) {
+    if (tempvarID >= skip) {
+      --tempvarID;
+    }
+  }
+
+  // Subtract if corrected importance and permuted variable, else add
+  if (importance_mode == IMP_GINI_CORRECTED && varID >= data->getNumCols()) {
+    (*variable_importance)[tempvarID] -= decrease;
+  } else {
+    (*variable_importance)[tempvarID] += decrease;
+  }
+}
